﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */

//Code is based on an implementation from Laurens van der Maaten

/*
*
* Copyright (c) 2014, Laurens van der Maaten (Delft University of Technology)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. All advertising materials mentioning features or use of this software
*    must display the following acknowledgement:
*    This product includes software developed by the Delft University of Technology.
* 4. Neither the name of the Delft University of Technology nor the names of
*    its contributors may be used to endorse or promote products derived from
*    this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY LAURENS VAN DER MAATEN ''AS IS'' AND ANY EXPRESS
* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL LAURENS VAN DER MAATEN BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
* BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
* OF SUCH DAMAGE.
*
*/
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  public class Cell : DeepCloneable, ICell {
    #region properties
    [Storable]
    private uint dimension;
    [Storable]
    private double[] corner;
    [Storable]
    private double[] width;
    #endregion

    #region HLConstructors & Cloning
    [StorableConstructor]
    protected Cell(bool deserializing) { }
    protected Cell(Cell original, Cloner cloner) : base(original, cloner) {
      dimension = original.dimension;
      corner = original.corner.Select(x => x).ToArray();
      width = original.width.Select(x => x).ToArray();
    }
    public override IDeepCloneable Clone(Cloner cloner) { return new Cell(this, cloner); }

    public Cell(uint inpDimension) {
      dimension = inpDimension;
      corner = new double[dimension];
      width = new double[dimension];
    }
    public Cell(uint inpDimension, IEnumerable<double> inpCorner, IEnumerable<double> inpWidth) {
      dimension = inpDimension;
      corner = inpCorner.Select(x => x).ToArray();
      width = inpWidth.Select(x => x).ToArray();
    }
    #endregion

    public double GetCorner(int d) {
      return corner[d];
    }
    public double GetWidth(int d) {
      return width[d];
    }
    public void SetCorner(int d, double val) {
      corner[d] = val;
    }
    public void SetWidth(int d, double val) {
      width[d] = val;
    }

    public bool ContainsPoint(double[] point) {
      for (var d = 0; d < dimension; d++)
        if (corner[d] - width[d] > point[d] || corner[d] + width[d] < point[d]) return false;
      return true;
    }
  }
}
